import 'package:flutter/material.dart';
import 'package:pit_components/components/adv_row.dart';
import 'package:pit_components/components/adv_visibility.dart';
import 'package:pit_components/pit_components.dart';

class AdvImagePreview extends StatefulWidget {
  final AdvImagePreviewController controller;

  AdvImagePreview(
      {int currentImage,
      List<Widget> images,
      AdvImagePreviewController controller})
      : assert(controller == null || (currentImage == null && images == null)),
        this.controller = controller ??
            AdvImagePreviewController(
                currentImage: currentImage ?? 0, images: images ?? const []);

  @override
  _AdvImagePreviewState createState() => new _AdvImagePreviewState();
}

class _AdvImagePreviewState extends State<AdvImagePreview> {
  @override
  void initState() {
    super.initState();

    widget.controller.addListener(() {
      setState(() {});
    });
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> children = [];
    final AdvImagePreviewController controller = widget.controller;

    children.add(Expanded(
        child: OverflowBox(
      child: controller.images[controller.currentImage],
      maxHeight: double.infinity,
    )));

    if (controller.images.length > 1) {
      List<Widget> thumbnails = [];

      for (int i = 0; i < controller.images.length; i++) {
        Widget image = controller.images[i];

        thumbnails.add(InkWell(
            onTap: () {
              controller.currentImage = i;
            },
            child: Stack(children: [
              image,
              AdvVisibility(
                child: Positioned(
                  bottom: 0.0,
                  left: 0.0,
                  right: 0.0,
                  height: 4.0,
                  child:
                      Container(color: PitComponents.selectedImagePreviewColor),
                ),
                visibility: i == controller.currentImage
                    ? VisibilityFlag.visible
                    : VisibilityFlag.gone,
              )
            ])));
      }

      Widget rowOfThumbnails = SingleChildScrollView(
        scrollDirection: Axis.horizontal,
        child: AdvRow(
            mainAxisSize: MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.start,
            padding: EdgeInsets.symmetric(vertical: 4.0),
            divider: RowDivider(4.0),
            children: thumbnails),
      );

      children.add(rowOfThumbnails);
    }

    return Column(
      children: children,
      mainAxisSize: MainAxisSize.max,
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    print("ImageWithThumbnail didChangeDependencies");
  }
}

class AdvImagePreviewController extends ValueNotifier {
  int get currentImage => value.currentImage;

  set currentImage(int newCurrentImage) {
    value =
        value.copyWith(currentImage: newCurrentImage, imageUrls: this.images);
  }

  List<Widget> get images => value.images;

  set images(List<Widget> newImages) {
    value = value.copyWith(currentImage: this.currentImage, images: newImages);
  }

  AdvImagePreviewController({int currentImage, List<Widget> images})
      : super(currentImage == null && images == null
            ? AdvImagePreviewEditingValue.empty
            : new AdvImagePreviewEditingValue(
                currentImage: currentImage, images: images));

  AdvImagePreviewController.fromValue(AdvImagePreviewEditingValue value)
      : super(value ?? AdvImagePreviewEditingValue.empty);

  void clear() {
    value = AdvImagePreviewEditingValue.empty;
  }
}

@immutable
class AdvImagePreviewEditingValue {
  const AdvImagePreviewEditingValue(
      {int currentImage, List<Widget> images = const []})
      : this.currentImage = currentImage ?? 0,
        this.images = images ?? const [];

  final int currentImage;
  final List<Widget> images;

  static const AdvImagePreviewEditingValue empty =
      const AdvImagePreviewEditingValue();

  AdvImagePreviewEditingValue copyWith(
      {int currentImage, List<Widget> images}) {
    return new AdvImagePreviewEditingValue(
        currentImage: currentImage ?? this.currentImage,
        images: images ?? this.images);
  }

  AdvImagePreviewEditingValue.fromValue(AdvImagePreviewEditingValue copy)
      : this.currentImage = copy.currentImage,
        this.images = copy.images;

  @override
  String toString() => '$runtimeType(currentImage: \u2524$currentImage\u251C, '
      'images: \u2524$images\u251C)';

  @override
  bool operator ==(dynamic other) {
    if (identical(this, other)) return true;
    if (other is! AdvImagePreviewEditingValue) return false;
    final AdvImagePreviewEditingValue typedOther = other;
    return typedOther.currentImage == currentImage &&
        typedOther.images == images;
  }

  @override
  int get hashCode => hashValues(currentImage.hashCode, images.hashCode);
}
